"""This module includes Tokenizer class for tokenizing texts"""
from abc import ABC
from typing import List, Tuple

from ...tokenizer import Tokenizer


class PuncTokenizer(Tokenizer, ABC):
    """
    A class impl tokenizer with punctuation restoration and then tokenize it
    """

    def word_span_tokenize(self, text: str) -> List[Tuple[int, int, str]]:
        pass

    def sentence_span_tokenize(self, text: str) -> List[Tuple[int, int, str]]:
        pass



# testing Fibonacci number function
# pylint: skip-file
from ..piraye import NltkTokenizer, SpacyTokenizer


def test_object():
    tokenizer = NltkTokenizer()
    assert tokenizer is not None


def test_sample():
    text = "برای تست شماره ۲.۱ نوشته شده است"
    tokenizer = NltkTokenizer()
    assert len(tokenizer.word_tokenize(text)) == 7


def test_sample_spacy():
    text = "برای تست (شماره ۲.۱ نوشته) شده است"
    tokenizer = SpacyTokenizer()
    assert len(tokenizer.word_tokenize(text)) == 9


def test_double_quotes():
    text = "'\"\"تست\""
    tokenizer = NltkTokenizer()
    assert len(tokenizer.word_tokenize(text)) == 5


def test_sentence_tokenizer():
    text = "sentence1 sad. \n asd asd \n asdasd \n sentence 2."
    tokenizer = NltkTokenizer()
    assert len(tokenizer.sentence_tokenize(text)) == 2
    assert len(tokenizer.sentence_span_tokenize(text)) == 2


def test_sentence_tokenizer_spacy():
    text = "sentence1 sad. \n asd asd \n asdasd \n sentence 2."
    tokenizer = SpacyTokenizer()
    assert len(tokenizer.sentence_tokenize(text)) == 2
    assert len(tokenizer.sentence_span_tokenize(text)) == 2


def test_double_quotes2():
    text = "«»"
    tokenizer = NltkTokenizer()
    assert len(tokenizer.word_tokenize(text)) == 2


def test_link():
    # To check nltk is functioning wrong for links
    text = "این یک لینک تست است https://www.google.com "
    tokenizer = NltkTokenizer()
    assert len(tokenizer.word_tokenize(text)) != 9


def test_link_spacy():
    text = "این یک لینک، (تست) است https://www.google.com "
    tokenizer = SpacyTokenizer()
    assert len(tokenizer.word_tokenize(text)) == 9



"""This module includes a Tokenizer class for tokenizing texts"""
from abc import ABC
from typing import List, Tuple
from spacy.lang.en import English
from spacy.pipeline import Sentencizer

from ..normalizer.mappings import MappingDict
from ...tokenizer import Tokenizer


class SpacyTokenizer(Tokenizer, ABC):
    """
    A class impl tokenizer with spacy
    ...
    Methods
    -------
    word_tokenize(text: str):
        return tokenized text
    sentence_tokenize(text: str):
        return sentence tokenized text
    """

    def __init__(self, ):
        """
        constructor
        """
        self.__nlp = English()
        self.__en_mapping = MappingDict.load_jsons(["digit_en", "punc_en"])
        self.__tokenizer = self.__nlp.tokenizer
        self.__sentencizer = Sentencizer()

    def word_span_tokenize(self, text) -> List[Tuple[int, int, str]]:
        text2 = self.__clean_text(text)
        spans = self.__tokenizer(text2)
        return [(span.idx, span.idx + len(span.text), text[span.idx: span.idx + len(span.text)]) for span in spans]

    def sentence_span_tokenize(self, text) -> List[Tuple[int, int, str]]:
        text2 = self.__clean_text(text)
        spans = self.__sentencizer(self.__nlp(text2))
        result = []
        last_index = 0
        for span in spans:
            if span.is_sent_end:
                result.append((last_index, span.idx + len(span.text), text[last_index:span.idx + len(span.text)]))
                last_index = span.idx + len(span.text)
        return result

    def __clean_text(self, text: str) -> str:
        """
        Clean the input text by replacing digits and punctuation with normalized versions.
        :param text: He inputs text to clean.
        :return: The cleaned text with normalized digits and punctuation.
        """
        return ''.join(
            [char if not self.__en_mapping.get(char)
             else self.__en_mapping.get(char).char for char in text])



# pylint: skip-file
from __future__ import annotations

from dataclasses import dataclass
from typing import Dict


@dataclass(unsafe_hash=False, frozen=True)
class CharConfig:
    char: str
    space_before: bool = None
    space_after: bool = None
    space_priority: int = None
    is_token: bool = None

    @property
    def is_space(self) -> str:
        return self.space_priority is not None

    @staticmethod
    def from_dict(data: Dict, config: str) -> CharConfig:
        return CharConfig(data["map"][config]["char"],
                          data["clean"].get('space_before') if data.get("clean") else None,
                          data["clean"].get('space_after') if data.get("clean") else None,
                          data["clean"].get('space_priority') if data.get("clean") else None,
                          data["clean"].get('is_token') if data.get("clean") else None
                          )



"""Base class of tokenizer"""
from abc import ABC, abstractmethod
from typing import List, Tuple

from ...normalizer import Normalizer


class PunctuationRestoration(Normalizer, ABC):
    """
    Normalize text by restoring mission punctuation
    """

    @abstractmethod
    def normalize(self, text: str) -> str:
        """
        returns a text with punctuation
        :param text: Input text without punctuation
        :return: The text with punctuation
        """

    @abstractmethod
    def span_normalize(self, text: str) -> List[Tuple[int, int, str]]:
        """
        Return spans of missed punctuations
        :param text: Input text without punctuation
        :return: A list of tuples containing the start index, end index, and punctuation token for each token span.
        """
